from django.conf import settings
from django.contrib.staticfiles.storage import CachedFilesMixin

from boto.utils import parse_ts
from storages.backends.s3boto import S3BotoStorage


class CachedStaticS3Storage(CachedFilesMixin, S3BotoStorage):
    """
    A storage class that allows for the speification of a second S3 bucket
    (in this case to give static media its own location), while also
    giving it the same post-processing functionality that Django's cached
    storage supplies.

    """
    def __init__(self, *args, **kwargs):
        kwargs['bucket'] = getattr(settings, 'STATIC_STORAGE_BUCKET_NAME')
        super(CachedStaticS3Storage, self).__init__(*args, **kwargs)

    def modified_time(self, name):
        name = self._normalize_name(self._clean_name(name))
        entry = self.entries.get(name)
        if entry is None:
            entry = self.bucket.get_key(self._encode_name(name))
        # Parse the last_modified string to a local datetime object.
        return parse_ts(entry.last_modified)



# -*- coding: utf-8 -*-

COUNTRY_CHOICES = (
  ('ad', 'Andorra, Principality of'),
  ('ae', 'United Arab Emirates'),
  ('af', 'Afghanistan, Islamic State of'),
  ('ag', 'Antigua and Barbuda'),
  ('ai', 'Anguilla'),
  ('al', 'Albania'),
  ('am', 'Armenia'),
  ('an', 'Netherlands Antilles'),
  ('ao', 'Angola'),
  ('aq', 'Antarctica'),
  ('ar', 'Argentina'),
  ('as', 'American Samoa'),
  ('at', 'Austria'),
  ('au', 'Australia'),
  ('aw', 'Aruba'),
  ('az', 'Azerbaidjan'),
  ('ba', 'Bosnia-Herzegovina'),
  ('bb', 'Barbados'),
  ('bd', 'Bangladesh'),
  ('be', 'Belgium'),
  ('bf', 'Burkina Faso'),
  ('bg', 'Bulgaria'),
  ('bh', 'Bahrain'),
  ('bi', 'Burundi'),
  ('bj', 'Benin'),
  ('bm', 'Bermuda'),
  ('bn', 'Brunei Darussalam'),
  ('bo', 'Bolivia'),
  ('br', 'Brazil'),
  ('bs', 'Bahamas'),
  ('bt', 'Bhutan'),
  ('bv', 'Bouvet Island'),
  ('bw', 'Botswana'),
  ('by', 'Belarus'),
  ('bz', 'Belize'),
  ('ca', 'Canada'),
  ('cc', 'Cocos (Keeling) Islands'),
  ('cf', 'Central African Republic'),
  ('cd', 'Congo, The Democratic Republic of the'),
  ('cg', 'Congo'),
  ('ch', 'Switzerland'),
  ('ci', 'Ivory Coast (Cote D\'Ivoire)'),
  ('ck', 'Cook Islands'),
  ('cl', 'Chile'),
  ('cm', 'Cameroon'),
  ('cn', 'China'),
  ('co', 'Colombia'),
  ('cr', 'Costa Rica'),
  ('cs', 'Former Czechoslovakia'),
  ('cu', 'Cuba'),
  ('cv', 'Cape Verde'),
  ('cx', 'Christmas Island'),
  ('cy', 'Cyprus'),
  ('cz', 'Czech Republic'),
  ('de', 'Germany'),
  ('dj', 'Djibouti'),
  ('dk', 'Denmark'),
  ('dm', 'Dominica'),
  ('do', 'Dominican Republic'),
  ('dz', 'Algeria'),
  ('ec', 'Ecuador'),
  ('ee', 'Estonia'),
  ('eg', 'Egypt'),
  ('eh', 'Western Sahara'),
  ('er', 'Eritrea'),
  ('es', 'Spain'),
  ('et', 'Ethiopia'),
  ('fi', 'Finland'),
  ('fj', 'Fiji'),
  ('fk', 'Falkland Islands'),
  ('fm', 'Micronesia'),
  ('fo', 'Faroe Islands'),
  ('fr', 'France'),
  ('fx', 'France (European Territory)'),
  ('ga', 'Gabon'),
  ('gb', 'Great Britain'),
  ('gd', 'Grenada'),
  ('ge', 'Georgia'),
  ('gf', 'French Guyana'),
  ('gh', 'Ghana'),
  ('gi', 'Gibraltar'),
  ('gl', 'Greenland'),
  ('gm', 'Gambia'),
  ('gn', 'Guinea'),
  ('gp', 'Guadeloupe (French)'),
  ('gq', 'Equatorial Guinea'),
  ('gr', 'Greece'),
  ('gs', 'S. Georgia & S. Sandwich Isls.'),
  ('gt', 'Guatemala'),
  ('gu', 'Guam (USA)'),
  ('gw', 'Guinea Bissau'),
  ('gy', 'Guyana'),
  ('hk', 'Hong Kong'),
  ('hm', 'Heard and McDonald Islands'),
  ('hn', 'Honduras'),
  ('hr', 'Croatia'),
  ('ht', 'Haiti'),
  ('hu', 'Hungary'),
  ('id', 'Indonesia'),
  ('ie', 'Ireland'),
  ('il', 'Israel'),
  ('in', 'India'),
  ('io', 'British Indian Ocean Territory'),
  ('iq', 'Iraq'),
  ('ir', 'Iran'),
  ('is', 'Iceland'),
  ('it', 'Italy'),
  ('jm', 'Jamaica'),
  ('jo', 'Jordan'),
  ('jp', 'Japan'),
  ('ke', 'Kenya'),
  ('kg', 'Kyrgyz Republic (Kyrgyzstan)'),
  ('kh', 'Cambodia, Kingdom of'),
  ('ki', 'Kiribati'),
  ('km', 'Comoros'),
  ('kn', 'Saint Kitts & Nevis Anguilla'),
  ('kp', 'North Korea'),
  ('kr', 'South Korea'),
  ('kw', 'Kuwait'),
  ('ky', 'Cayman Islands'),
  ('kz', 'Kazakhstan'),
  ('la', 'Laos'),
  ('lb', 'Lebanon'),
  ('lc', 'Saint Lucia'),
  ('li', 'Liechtenstein'),
  ('lk', 'Sri Lanka'),
  ('lr', 'Liberia'),
  ('ls', 'Lesotho'),
  ('lt', 'Lithuania'),
  ('lu', 'Luxembourg'),
  ('lv', 'Latvia'),
  ('ly', 'Libya'),
  ('ma', 'Morocco'),
  ('mc', 'Monaco'),
  ('md', 'Moldavia'),
  ('mg', 'Madagascar'),
  ('mh', 'Marshall Islands'),
  ('mk', 'Macedonia'),
  ('ml', 'Mali'),
  ('mm', 'Myanmar'),
  ('mn', 'Mongolia'),
  ('mo', 'Macau'),
  ('mp', 'Northern Mariana Islands'),
  ('mq', 'Martinique (French)'),
  ('mr', 'Mauritania'),
  ('ms', 'Montserrat'),
  ('mt', 'Malta'),
  ('mu', 'Mauritius'),
  ('mv', 'Maldives'),
  ('mw', 'Malawi'),
  ('mx', 'Mexico'),
  ('my', 'Malaysia'),
  ('mz', 'Mozambique'),
  ('na', 'Namibia'),
  ('nc', 'New Caledonia (French)'),
  ('ne', 'Niger'),
  ('nf', 'Norfolk Island'),
  ('ng', 'Nigeria'),
  ('ni', 'Nicaragua'),
  ('nl', 'Netherlands'),
  ('no', 'Norway'),
  ('np', 'Nepal'),
  ('nr', 'Nauru'),
  ('nt', 'Neutral Zone'),
  ('nu', 'Niue'),
  ('nz', 'New Zealand'),
  ('om', 'Oman'),
  ('pa', 'Panama'),
  ('pe', 'Peru'),
  ('pf', 'Polynesia (French)'),
  ('pg', 'Papua New Guinea'),
  ('ph', 'Philippines'),
  ('pk', 'Pakistan'),
  ('pl', 'Poland'),
  ('pm', 'Saint Pierre and Miquelon'),
  ('pn', 'Pitcairn Island'),
  ('pr', 'Puerto Rico'),
  ('pt', 'Portugal'),
  ('pw', 'Palau'),
  ('py', 'Paraguay'),
  ('qa', 'Qatar'),
  ('re', 'Reunion (French)'),
  ('ro', 'Romania'),
  ('ru', 'Russian Federation'),
  ('rw', 'Rwanda'),
  ('sa', 'Saudi Arabia'),
  ('sb', 'Solomon Islands'),
  ('sc', 'Seychelles'),
  ('sd', 'Sudan'),
  ('se', 'Sweden'),
  ('sg', 'Singapore'),
  ('sh', 'Saint Helena'),
  ('si', 'Slovenia'),
  ('sj', 'Svalbard and Jan Mayen Islands'),
  ('sk', 'Slovak Republic'),
  ('sl', 'Sierra Leone'),
  ('sm', 'San Marino'),
  ('sn', 'Senegal'),
  ('so', 'Somalia'),
  ('sr', 'Suriname'),
  ('st', 'Saint Tome (Sao Tome) and Principe'),
  ('su', 'Former USSR'),
  ('sv', 'El Salvador'),
  ('sy', 'Syria'),
  ('sz', 'Swaziland'),
  ('tc', 'Turks and Caicos Islands'),
  ('td', 'Chad'),
  ('tf', 'French Southern Territories'),
  ('tg', 'Togo'),
  ('th', 'Thailand'),
  ('tj', 'Tadjikistan'),
  ('tk', 'Tokelau'),
  ('tm', 'Turkmenistan'),
  ('tn', 'Tunisia'),
  ('to', 'Tonga'),
  ('tp', 'East Timor'),
  ('tr', 'Turkey'),
  ('tt', 'Trinidad and Tobago'),
  ('tv', 'Tuvalu'),
  ('tw', 'Taiwan'),
  ('tz', 'Tanzania'),
  ('ua', 'Ukraine'),
  ('ug', 'Uganda'),
  ('uk', 'United Kingdom'),
  ('um', 'USA Minor Outlying Islands'),
  ('us', 'United States'),
  ('uy', 'Uruguay'),
  ('uz', 'Uzbekistan'),
  ('va', 'Holy See (Vatican City State)'),
  ('vc', 'Saint Vincent & Grenadines'),
  ('ve', 'Venezuela'),
  ('vg', 'Virgin Islands (British)'),
  ('vi', 'Virgin Islands (USA)'),
  ('vn', 'Vietnam'),
  ('vu', 'Vanuatu'),
  ('wf', 'Wallis and Futuna Islands'),
  ('ws', 'Samoa'),
  ('ye', 'Yemen'),
  ('yt', 'Mayotte'),
  ('yu', 'Yugoslavia'),
  ('za', 'South Africa'),
  ('zm', 'Zambia'),
  ('zr', 'Zaire'),
  ('zw', 'Zimbabwe'),
)

JP_PREFECTURES = (
    ('hokkaidou', 'Hokkaidou'),
    ('aomori', 'Aomori'),
    ('iwate', 'Iwate'),
    ('miyagi', 'Miyagi'),
    ('akita', 'Akita'),
    ('yamagata', 'Yamagata'),
    ('fukushima', 'Fukushima'),
    ('ibaraki', 'Ibaraki'),
    ('tochigi', 'Tochigi'),
    ('gunma', 'Gunma'),
    ('saitama', 'Saitama'),
    ('chiba', 'Chiba'),
    ('toukyou', 'Toukyou'),
    ('kanagawa', 'Kanagawa'),
    ('niigata', 'Niigata'),
    ('toyama', 'Toyama'),
    ('ishikawa', 'Ishikawa'),
    ('fukui', 'Fukui'),
    ('yamanashi', 'Yamanashi'),
    ('nagano', 'Nagano'),
    ('gifu', 'Gifu'),
    ('shizuoka', 'Shizuoka'),
    ('aichi', 'Aichi'),
    ('mie', 'Mie'),
    ('shiga', 'Shiga'),
    ('kyouto', 'Kyouto'),
    ('oosaka', 'Oosaka'),
    ('hyougo', 'Hyougo'),
    ('nara', 'Nara'),
    ('wakayama', 'Wakayama'),
    ('tottori', 'Tottori'),
    ('shimane', 'Shimane'),
    ('okayama', 'Okayama'),
    ('hiroshima', 'Hiroshima'),
    ('yamaguchi', 'Yamaguchi'),
    ('tokushima', 'Tokushima'),
    ('kagawa', 'Kagawa'),
    ('ehime', 'Ehime'),
    ('kouchi', 'Kouchi'),
    ('fukuoka', 'Fukuoka'),
    ('saga', 'Saga'),
    ('nagasaki', 'Nagasaki'),
    ('kumamoto', 'Kumamoto'),
    ('ooita', 'Ooita'),
    ('miyazaki', 'Miyazaki'),
    ('kagoshima', 'Kagoshima'),
    ('okinawa', 'Okinawa'),
)

JP_PREFECTURES_KANJI = (
    ('hokkaidou', '北海道'),
    ('aomori', '青森県'),
    ('iwate', '岩手県'),
    ('miyagi', '宮城県'),
    ('akita', '秋田県'),
    ('yamagata', '山形県'),
    ('fukushima', '福島県'),
    ('ibaraki', '茨城県'),
    ('tochigi', '栃木県'),
    ('gunma', '群馬県'),
    ('saitama', '埼玉県'),
    ('chiba', '千葉県'),
    ('toukyou', '東京都'),
    ('kanagawa', '神奈川県'),
    ('niigata', '新潟県'),
    ('toyama', '富山県'),
    ('ishikawa', '石川県'),
    ('fukui', '福井県'),
    ('yamanashi', '山梨県'),
    ('nagano', '長野県'),
    ('gifu', '岐阜県'),
    ('shizuoka', '静岡県'),
    ('aichi', '愛知県'),
    ('mie', '三重県'),
    ('shiga', '滋賀県'),
    ('kyouto', '京都府'),
    ('oosaka', '大阪府'),
    ('hyougo', '兵庫県'),
    ('nara', '奈良県'),
    ('wakayama', '和歌山県'),
    ('tottori', '鳥取県'),
    ('shimane', '島根県'),
    ('okayama', '岡山県'),
    ('hiroshima', '広島県'),
    ('yamaguchi', '山口県'),
    ('tokushima', '徳島県'),
    ('kagawa', '香川県'),
    ('ehime', '愛媛県'),
    ('kouchi', '高知県'),
    ('fukuoka', '福岡県'),
    ('saga', '佐賀県'),
    ('nagasaki', '長崎県'),
    ('kumamoto', '熊本県'),
    ('ooita', '大分県'),
    ('miyazaki', '宮崎県'),
    ('kagoshima', '鹿児島県'),
    ('okinawa', '沖縄県'),
)

OTHER = 99



# -*- coding: utf-8 -*-
import uuid

from django.db.models import Field, SubfieldBase
from django.utils.encoding import force_bytes
from django.utils import six

from psycopg2.extensions import register_adapter


class UUIDAdapter(object):
    def __init__(self, value):
        if not isinstance(value, uuid.UUID):
            raise TypeError('UUIDAdapter only understands UUID objects.')
        self.value = value

    def getquoted(self):
        return force_bytes(u"'{0}'".format(self.value))


class UUIDField(six.with_metaclass(SubfieldBase, Field)):
    description = 'Universally unique identifier.'

    def __init__(self, auto_add=False, *args, **kwargs):
        self._auto_add = auto_add

        kwargs.setdefault('unique', True)
        kwargs.setdefault('blank', False)

        if auto_add:
            kwargs.setdefault('editable', False)

        if kwargs['blank'] and not kwargs.get('null', False):
            raise AttributeError('Blank UUIDs are stored as NULL. Therefore, setting '
                                 '`blank` to True requires `null` to be True.')

        super(UUIDField, self).__init__(*args, **kwargs)

    def db_type(self, connection):
        return 'uuid'

    def get_prep_value(self, value):
        if not value:
            if self.null or self._auto_add:
                return None

            raise ValueError('Explicit UUID required unless either '
                             '`null` or `auto_add` are True.')

        if isinstance(value, uuid.UUID):
            return value

        return uuid.UUID(value)

    def get_db_prep_value(self, value, connection, prepared=False):
        register_adapter(uuid.UUID, UUIDAdapter)
        return super(UUIDField, self).get_db_prep_value(
                     value, connection, prepared=prepared)

    def pre_save(self, instance, add):
        if self._auto_add and add and not getattr(instance, self.attname):
            random_uuid = uuid.uuid4()
            setattr(instance, self.attname, random_uuid)
            return random_uuid

        return super(UUIDField, self).pre_save(instance, add)

    def to_python(self, value):
        if isinstance(value, uuid.UUID) or not value:
            return value
        return uuid.UUID(value)



from django.db.models import *

from ohashi.db.models.fields import *
from ohashi.db.models.fields.birthdays import BirthdayField
from ohashi.db.models.fields.uuids import UUIDField
from ohashi.db.models.managers.birthdays import BirthdayManager
